<?php
/**
 * class-woocommerce-product-search-worker.php
 *
 * Copyright (c) "kento" Karim Rahimpur www.itthinx.com
 *
 * This code is provided subject to the license granted.
 * Unauthorized use and distribution is prohibited.
 * See COPYRIGHT.txt and LICENSE.txt
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This header and all notices must be kept intact.
 *
 * @author itthinx
 * @package woocommerce-product-search
 * @since 2.0.0
 */

if ( !defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Index worker.
 */
class WooCommerce_Product_Search_Worker {

	const INDEX_WORKER_STATUS          = 'index-worker-status';
	const INDEX_WORKER_STATUS_DEFAULT  = false;
	const START_DELTA                  = 1;
	const WORK_CYCLE                   = 'work-cycle';
	const WORK_CYCLE_DEFAULT           = 60;
	const IDLE_CYCLE                   = 'idle-cycle';
	const IDLE_CYCLE_DEFAULT           = 300;
	const LIMIT_PER_WORK_CYCLE         = 'limit-per-work-cycle';
	const LIMIT_PER_WORK_CYCLE_DEFAULT = 40;

	/**
	 * Initialize
	 */
	public static function init() {


		add_action( 'woocommerce_product_search_work', array( __CLASS__, 'work' ), 10, 0 );
		self::schedule();
		add_action( 'woocommerce_product_search_deactivate', array( __CLASS__, 'deactivate' ) );
	}

	/**
	 * Worker start
	 */
	public static function start() {


		wps_log_info( 'Worker is starting.' );
		$options = get_option( 'woocommerce-product-search', array() );
		$options[self::INDEX_WORKER_STATUS] = true;
		update_option( 'woocommerce-product-search', $options );
		self::schedule( self::START_DELTA );
	}

	/**
	 * Worker stop
	 */
	public static function stop() {


		wps_log_info( 'Worker is stopping.' );
		$options = get_option( 'woocommerce-product-search', array() );
		$options[self::INDEX_WORKER_STATUS] = false;
		update_option( 'woocommerce-product-search', $options );
		self::wp_unschedule_hook( 'woocommerce_product_search_work' );
	}

	/**
	 * Worker schedule
	 */
	private static function schedule( $delta = null ) {


		$options = get_option( 'woocommerce-product-search', array() );
		$status = isset( $options[self::INDEX_WORKER_STATUS] ) ? $options[self::INDEX_WORKER_STATUS] : self::INDEX_WORKER_STATUS_DEFAULT;

		if ( $status ) {

			if ( $delta !== null && $delta >= 0 ) {
				self::wp_unschedule_hook( 'woocommerce_product_search_work' );
			}
			if ( !self::get_next_scheduled() ) {
				$indexer = new WooCommerce_Product_Search_Indexer();
				$processable = $indexer->get_processable_count();
				$total       = $indexer->get_total_count();
				if ( $processable > 0 ) {
					if ( $delta !== null ) {
						$delta = intval( $delta );
						if ( $delta <= 0 ) {
							$delta = null;
						}
					}
					if ( $delta === null ) {
						$next = time() + self::get_work_cycle();
					} else {
						$next = time() + $delta;
					}
				} else {
					$next = time() + self::get_idle_cycle();
				}
				$scheduled = wp_schedule_single_event( $next, 'woocommerce_product_search_work' );
				wps_log_info( sprintf(
					'Worker @ %s; next scheduled @ %s',
					date( 'Y-m-d H:i:s', time() ),
					date( 'Y-m-d H:i:s', $next ) )
				);
				if ( $scheduled === false ) {
					wps_log_error( 'Worker could not schedule next work cycle.' );
				}
			}
		}
	}

	/**
	 * Worker deactivate
	 */
	public static function deactivate() {


		self::wp_unschedule_hook( 'woocommerce_product_search_work' );
	}

	/**
	 * Worker work
	 */
	public static function work() {


		$indexer = new WooCommerce_Product_Search_Indexer();
		$c = $indexer->get_processable_count();
		if ( $c > 0 ) {
			$indexer->work();
		}
		$indexer->gc();
		unset( $indexer );
	}

	/**
	 * Worker status
	 *
	 * @return boolean status
	 */
	public static function get_status() {


		$options = get_option( 'woocommerce-product-search', array() );
		return isset( $options[self::INDEX_WORKER_STATUS] ) ? $options[self::INDEX_WORKER_STATUS] : self::INDEX_WORKER_STATUS_DEFAULT;
	}

	/**
	 * Worker next scheduled
	 *
	 * @return int|false
	 */
	public static function get_next_scheduled() {


		return wp_next_scheduled( 'woocommerce_product_search_work' );
	}

	/**
	 * Worker work cycle
	 *
	 * @return int
	 */
	public static function get_work_cycle() {


		$options = get_option( 'woocommerce-product-search', array() );
		$max_execution_time = self::get_work_cycle_default();
		return intval( isset( $options[self::WORK_CYCLE] ) ? $options[self::WORK_CYCLE] : $max_execution_time );
	}

	public static function get_work_cycle_default() {
		$max_execution_time = intval( ini_get( 'max_execution_time' ) );
		if ( $max_execution_time <= 0 ) {
			$max_execution_time = self::WORK_CYCLE_DEFAULT;
		}
		return $max_execution_time;
	}

	/**
	 * Worker idle cycle
	 *
	 * @return int
	 */
	public static function get_idle_cycle() {


		$options = get_option( 'woocommerce-product-search', array() );
		return intval( isset( $options[self::IDLE_CYCLE] ) ? $options[self::IDLE_CYCLE] : self::IDLE_CYCLE_DEFAULT );
	}

	/**
	 * Worker limit
	 *
	 * @return int limit per cycle
	 */
	public static function get_limit_per_work_cycle() {


		$options = get_option( 'woocommerce-product-search', array() );
		return intval( isset( $options[self::LIMIT_PER_WORK_CYCLE] ) ? $options[self::LIMIT_PER_WORK_CYCLE] : self::LIMIT_PER_WORK_CYCLE_DEFAULT );
	}

	/**
	 * Unschedules all events attached to the hook.
	 *
	 * This function is available as of WP 4.9.0 - added here for backwards compatibility.
	 *
	 * @param string $hook Action hook, the execution of which will be unscheduled.
	 */
	public static function wp_unschedule_hook( $hook ) {
		if ( function_exists( 'wp_unschedule_hook' ) ) {
			wp_unschedule_hook( 'woocommerce_product_search_work' );
		} else {
			$crons = _get_cron_array();
			foreach( $crons as $timestamp => $args ) {
				unset( $crons[ $timestamp ][ $hook ] );
				if ( empty( $crons[ $timestamp ] ) ) {
					unset( $crons[ $timestamp ] );
				}
			}
			_set_cron_array( $crons );
		}
	}
}
WooCommerce_Product_Search_Worker::init();
