<?php
/**
 * class-woocommerce-product-search-taxonomy-renderer.php
 *
 * Copyright (c) "kento" Karim Rahimpur www.itthinx.com
 *
 * This code is provided subject to the license granted.
 * Unauthorized use and distribution is prohibited.
 * See COPYRIGHT.txt and LICENSE.txt
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This header and all notices must be kept intact.
 *
 * @author itthinx
 * @package woocommerce-product-search
 * @since 2.0.4
 */

if ( !defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Taxonomy renderer
 *
 * @access private
 */
class WooCommerce_Product_Search_Taxonomy_Renderer {

	public function __construct() {
	}

	public function list_categories( $args = '' ) {
		$defaults = array(
			'child_of'            => 0,
			'current_category'    => 0,
			'depth'               => 0,
			'echo'                => 1,
			'exclude'             => '',
			'exclude_tree'        => '',
			'feed'                => '',
			'feed_image'          => '',
			'feed_type'           => '',
			'hide_empty'          => 1,
			'hide_title_if_empty' => false,
			'hierarchical'        => true,
			'order'               => 'ASC',
			'orderby'             => 'name',
			'separator'           => '<br />',
			'show_count'          => 0,
			'show_option_all'     => '',
			'show_option_none'    => __( 'No categories' ),
			'style'               => 'list',
			'taxonomy'            => 'category',
			'title_li'            => __( 'Categories' ),
			'use_desc_for_title'  => 1,
		);

		$r = wp_parse_args( $args, $defaults );

		if ( !isset( $r['pad_counts'] ) && $r['show_count'] && $r['hierarchical'] )
			$r['pad_counts'] = true;

		if ( true == $r['hierarchical'] ) {
			$exclude_tree = array();

			if ( $r['exclude_tree'] ) {
				$exclude_tree = array_merge( $exclude_tree, wp_parse_id_list( $r['exclude_tree'] ) );
			}

			if ( $r['exclude'] ) {
				$exclude_tree = array_merge( $exclude_tree, wp_parse_id_list( $r['exclude'] ) );
			}

			$r['exclude_tree'] = $exclude_tree;
			$r['exclude'] = '';
		}

		if ( ! isset( $r['class'] ) )
			$r['class'] = ( 'category' == $r['taxonomy'] ) ? 'categories' : $r['taxonomy'];

		if ( ! taxonomy_exists( $r['taxonomy'] ) ) {
			return false;
		}

		$show_option_all = $r['show_option_all'];
		$show_option_none = $r['show_option_none'];

		$categories = $this->get_categories( $r );

		$output = '';
		if ( $r['title_li'] && 'list' == $r['style'] && ( ! empty( $categories ) || ! $r['hide_title_if_empty'] ) ) {
			$output = '<li class="' . esc_attr( $r['class'] ) . '">' . $r['title_li'] . '<ul>';
		}
		if ( empty( $categories ) ) {
			if ( ! empty( $show_option_none ) ) {
				if ( 'list' == $r['style'] ) {
					$output .= '<li class="cat-item-none">' . $show_option_none . '</li>';
				} else {
					$output .= $show_option_none;
				}
			}
		} else {
			if ( ! empty( $show_option_all ) ) {

				$posts_page = '';

				$taxonomy_object = get_taxonomy( $r['taxonomy'] );
				if ( ! in_array( 'post', $taxonomy_object->object_type ) && ! in_array( 'page', $taxonomy_object->object_type ) ) {
					foreach ( $taxonomy_object->object_type as $object_type ) {
						$_object_type = get_post_type_object( $object_type );

						if ( ! empty( $_object_type->has_archive ) ) {
							$posts_page = get_post_type_archive_link( $object_type );
							break;
						}
					}
				}

				if ( ! $posts_page ) {
					if ( 'page' == get_option( 'show_on_front' ) && get_option( 'page_for_posts' ) ) {
						$posts_page = get_permalink( get_option( 'page_for_posts' ) );
					} else {
						$posts_page = home_url( '/' );
					}
				}

				$posts_page = esc_url( $posts_page );
				if ( 'list' == $r['style'] ) {
					$output .= "<li class='cat-item-all'><a href='$posts_page'>$show_option_all</a></li>";
				} else {
					$output .= "<a href='$posts_page'>$show_option_all</a>";
				}
			}

			if ( empty( $r['current_category'] ) && ( is_category() || is_tax() || is_tag() ) ) {
				$current_term_object = get_queried_object();
				if ( $current_term_object && $r['taxonomy'] === $current_term_object->taxonomy ) {
					$r['current_category'] = get_queried_object_id();
				}
			}

			if ( $r['hierarchical'] ) {
				$depth = $r['depth'];
			} else {
				$depth = -1; 
			}
			$output .= walk_category_tree( $categories, $depth, $r );
		}

		if ( $r['title_li'] && 'list' == $r['style'] && ( ! empty( $categories ) || ! $r['hide_title_if_empty'] ) ) {
			$output .= '</ul></li>';
		}

		/**
		 * Filters the HTML output of a taxonomy list.
		 *
		 * @since 2.1.0
		 *
		 * @param string $output HTML output.
		 * @param array  $args   An array of taxonomy-listing arguments.
		 */
		$html = apply_filters( 'wp_list_categories', $output, $args );

		if ( $r['echo'] ) {
			echo $html;
		} else {
			return $html;
		}
	}

	/**
	 * Retrieve list of category objects.
	 *
	 * If you change the type to 'link' in the arguments, then the link categories
	 * will be returned instead. Also all categories will be updated to be backward
	 * compatible with pre-2.3 plugins and themes.
	 *
	 * @since 2.1.0
	 * @see get_terms() Type of arguments that can be changed.
	 *
	 * @param string|array $args {
	 *     Optional. Arguments to retrieve categories. See get_terms() for additional options.
	 *
	 *     @type string $taxonomy Taxonomy to retrieve terms for. In this case, default 'category'.
	 * }
	 * @return array List of categories.
	 */
	private function get_categories( $args = '' ) {
		$defaults = array( 'taxonomy' => 'category' );
		$args = wp_parse_args( $args, $defaults );

		$taxonomy = $args['taxonomy'];

		/**
		 * Filters the taxonomy used to retrieve terms when calling get_categories().
		 *
		 * @since 2.7.0
		 *
		 * @param string $taxonomy Taxonomy to retrieve terms from.
		 * @param array  $args     An array of arguments. See get_terms().
		 */
		$taxonomy = apply_filters( 'get_categories_taxonomy', $taxonomy, $args );

		if ( isset($args['type']) && 'link' == $args['type'] ) {
			_deprecated_argument( __FUNCTION__, '3.0.0',
				
				sprintf( __( '%1$s is deprecated. Use %2$s instead.' ),
					'<code>type => link</code>',
					'<code>taxonomy => link_category</code>'
				)
			);
			$taxonomy = $args['taxonomy'] = 'link_category';
		}

		$categories = get_terms( $taxonomy, $args );

		if ( is_wp_error( $categories ) ) {
			$categories = array();
		} else {
			$categories = (array) $categories;
			foreach ( array_keys( $categories ) as $k ) {
				_make_cat_compat( $categories[ $k ] );
			}
		}
	
		return $categories;
	}
}
